// Copyright (c) 2018 Josef Spidlen, Ph.D., FlowJo, LLC
//
// License
// The software is distributed under the terms of the 
// Artistic License 2.0
// http://www.r-project.org/Licenses/Artistic-2.0
// 
// Disclaimer
// This software and documentation come with no warranties of any kind.
// This software is provided "as is" and any express or implied 
// warranties, including, but not limited to, the implied warranties of
// merchantability and fitness for a particular purpose are disclaimed.
// In no event shall the  copyright holder be liable for any direct, 
// indirect, incidental, special, exemplary, or consequential damages
// (including but not limited to, procurement of substitute goods or 
// services; loss of use, data or profits; or business interruption)
// however caused and on any theory of liability, whether in contract,
// strict liability, or tort arising in any way out of the use of this 
// software.    
//////////////////////////////////////////////////////////////////////////////

package com.flowjo.plugin.alra;

import java.io.BufferedReader;
import java.io.File;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.StringWriter;
import java.util.List;

import com.treestar.flowjo.engine.EngineManager;
import com.treestar.flowjo.engine.utility.RFlowCalculator;
import com.treestar.lib.file.FileUtil;

public class ALRACalc extends RFlowCalculator {
	
	private final static String scriptTemplatePath = "r/RScript.ALRA.Template.R";
	
	public File runCalc(File sampleFile, String sampleName, List<String> parameterNames, int k, int q, boolean normalize, String outputFolderPath)
    {
		sampleName = sampleName.replaceAll(".ExtNode", "").replaceAll(".fcs", "").replaceAll(".LMD", "").trim();
        File outputFolder = new File(outputFolderPath);
        StringWriter scriptWriter = new StringWriter();
        File rScriptOutput = createScript(sampleFile, sampleName, parameterNames, k, q, normalize, outputFolder, scriptWriter);
        if(rScriptOutput == null) return null;        
        String scriptFileName = (new StringBuilder()).append("RScript.ALRA.").append(System.currentTimeMillis()).append(".R").toString().replaceAll(" ", "_");
        try
        {
            fScriptFile = new File(outputFolderPath, scriptFileName);
            FileUtil.write(fScriptFile, scriptWriter.toString());
            executeRBatch(scriptFileName);
        }
        catch(Exception e) 
        {
            e.printStackTrace();
        }
        return rScriptOutput;
    }

	private File createScript(File sampleFile, String sampleName, List<String> parameterNames, int k, int q, boolean normalize, File outputFolder, StringWriter scriptWriter)
	{
        InputStream scriptStream = ALRACalc.class.getResourceAsStream(scriptTemplatePath);
        
        String outFileName = (new StringBuilder()).append(fixFileName(sampleName)).append(".ALRA").append(".csv").toString();
        if(outputFolder == null) outputFolder = sampleFile.getParentFile();
        
        File outFile = new File(outputFolder, outFileName);
        outFileName = outFile.getAbsolutePath();
        String dataFilePath = sampleFile.getAbsolutePath();

        if(EngineManager.isWindows()) outFileName = outFileName.replaceAll("\\\\", "/");
        if(EngineManager.isWindows()) dataFilePath = dataFilePath.replaceAll("\\\\", "/");

        BufferedReader rTemplateReader = null;
        try {
			rTemplateReader = new BufferedReader(new InputStreamReader(scriptStream));
		} catch (Exception e) {
			e.printStackTrace();
			return null;
		}
                
        String scriptLine;
        try {
			while((scriptLine = rTemplateReader.readLine()) != null) 
			{	
			    scriptLine = scriptLine.replaceAll("FJ_DATA_FILE_PATH", dataFilePath);
			    scriptLine = scriptLine.replaceAll("FJ_CSV_OUPUT_FILE", outFileName);
			    scriptLine = scriptLine.replaceAll("FJ_PAR_K", "" + k);
			    scriptLine = scriptLine.replaceAll("FJ_PAR_Q", "" + q);
			    scriptLine = scriptLine.replaceAll("FJ_PAR_NORMALIZE", normalize ? "TRUE" : "FALSE");
			    			    
			    if (scriptLine.contains("FJ_PARAMS_LIST")) {
			    	String parListStr = String.join("\",\"", parameterNames);
			    	scriptLine = scriptLine.replaceAll("FJ_PARAMS_LIST", "\"" + parListStr + "\"");
			    }
			    scriptWriter.append(scriptLine).append('\n');
			}
		} catch (Exception e) {
			e.printStackTrace();
		}
        
        if(rTemplateReader != null) {
        	try { rTemplateReader.close(); }
        	catch (Exception e) { e.printStackTrace(); }
        }
        
        return outFile;
	}

	
	public static String fixFileName(String fileName) {
		return fileName.
			replaceAll(" ", "_").replaceAll("/", "_").replaceAll("\\\\", "_").
	        replaceAll("\\:", "_").replaceAll("\\*", "_").replaceAll("\\?", "_").
	        replaceAll("\"", "_").replaceAll("\\|", "_");
	}
}
